@@ -27,6 +27,7 @@ from calibre.utils.config import tweaks
 
 pretty_print_opf = False
 
+
 class PrettyPrint(object):
 
     def __enter__(self):
@@ -36,8 +37,11 @@ class PrettyPrint(object):
     def __exit__(self, *args):
         global pretty_print_opf
         pretty_print_opf = False
+
+
 pretty_print = PrettyPrint()
 
+
 class Resource(object):  # {{{
 
     '''
@@ -121,6 +125,7 @@ class Resource(object):  # {{{
 
 # }}}
 
+
 class ResourceCollection(object):  # {{{
 
     def __init__(self):
@@ -174,6 +179,7 @@ class ResourceCollection(object):  # {{{
 
 # }}}
 
+
 class ManifestItem(Resource):  # {{{
 
     @staticmethod
@@ -190,6 +196,7 @@ class ManifestItem(Resource):  # {{{
     def media_type(self):
         def fget(self):
             return self.mime_type
+
         def fset(self, val):
             self.mime_type = val
         return property(fget=fget, fset=fset)
@@ -212,6 +219,7 @@ class ManifestItem(Resource):  # {{{
 
 # }}}
 
+
 class Manifest(ResourceCollection):  # {{{
 
     def append_from_opf_manifest_item(self, item, dir):
@@ -283,6 +291,7 @@ class Manifest(ResourceCollection):  # {{{
 
 # }}}
 
+
 class Spine(ResourceCollection):  # {{{
 
     class Item(Resource):
@@ -357,6 +366,7 @@ class Spine(ResourceCollection):  # {{{
 
 # }}}
 
+
 class Guide(ResourceCollection):  # {{{
 
     class Reference(Resource):
@@ -395,6 +405,7 @@ class Guide(ResourceCollection):  # {{{
 
 # }}}
 
+
 class MetadataField(object):
 
     def __init__(self, name, is_dc=True, formatter=None, none_is=None,
@@ -437,6 +448,7 @@ class MetadataField(object):
             elem = obj.create_metadata_element(self.name, is_dc=self.is_dc)
         obj.set_text(elem, self.renderer(val))
 
+
 class TitleSortField(MetadataField):
 
     def __get__(self, obj, type=None):
@@ -465,6 +477,7 @@ class TitleSortField(MetadataField):
                     if attr.endswith('file-as'):
                         del match.attrib[attr]
 
+
 def serialize_user_metadata(metadata_elem, all_user_metadata, tail='\n'+(' '*8)):
     from calibre.utils.config import to_json
     from calibre.ebooks.metadata.book.json_codec import (object_to_unicode,
@@ -495,6 +508,7 @@ def dump_dict(cats):
     return json.dumps(object_to_unicode(cats), ensure_ascii=False,
             skipkeys=True)
 
+
 class OPF(object):  # {{{
 
     MIMETYPE         = 'application/oebps-package+xml'
@@ -620,6 +634,9 @@ class OPF(object):  # {{{
         self._user_metadata_ = temp.get_all_user_metadata(True)
 
     def to_book_metadata(self):
+        if self.package_version >= 3.0:
+            from calibre.ebooks.metadata.opf3 import read_metadata
+            return read_metadata(self.root)
         ans = MetaInformation(self)
         for n, v in self._user_metadata_.items():
             ans.set_user_metadata(n, v)
@@ -1280,6 +1297,7 @@ class OPF(object):  # {{{
             smap[child.get('name')] = (child, self.metadata.index(child))
         if len(smap) == 2 and smap['calibre:series'][1] > smap['calibre:series_index'][1]:
             s, si = smap['calibre:series'][0], smap['calibre:series_index'][0]
+
             def swap(attr):
                 t = s.get(attr, '')
                 s.set(attr, si.get(attr, '')), si.set(attr, t)
@@ -1339,6 +1357,7 @@ class OPF(object):  # {{{
 
 # }}}
 
+
 class OPFCreator(Metadata):
 
     def __init__(self, base_path, other):
@@ -1553,6 +1572,7 @@ class OPFCreator(Metadata):
                 guide
         )
         root.set('unique-identifier', __appname__+'_id')
+        root.set('version', '2.0')
         raw = etree.tostring(root, pretty_print=True, xml_declaration=True,
                 encoding=encoding)
         raw = raw.replace(DNS, OPF2_NS)
@@ -1596,6 +1616,7 @@ def metadata_to_opf(mi, as_string=True, default_lang=None):
     metadata = root[0]
     guide = root[1]
     metadata[0].tail = '\n'+(' '*8)
+
     def factory(tag, text=None, sort=None, role=None, scheme=None, name=None,
             content=None):
         attrib = {}
@@ -1783,12 +1804,15 @@ class OPFTest(unittest.TestCase):
         self.opf.smart_update(MetaInformation(self.opf))
         self.testReading()
 
+
 def suite():
     return unittest.TestLoader().loadTestsFromTestCase(OPFTest)
 
+
 def test():
     unittest.TextTestRunner(verbosity=2).run(suite())
 
+
 def test_user_metadata():
     from cStringIO import StringIO
     mi = Metadata('Test title', ['test author1', 'test author2'])
@@ -1814,6 +1838,7 @@ def test_user_metadata():
     assert um == opf2._user_metadata_
     print opf.render()
 
+
 if __name__ == '__main__':
     # test_user_metadata()
     test_m2o()
