@@ -305,7 +305,7 @@ Copyright and License
 SmartyPants_ license::
 
     Copyright (c) 2003 John Gruber
-    (http://daringfireball.net/)
+    (https://daringfireball.net/)
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -367,11 +367,11 @@ smartypants.py license::
 
 
 
-.. _John Gruber: http://daringfireball.net/
+.. _John Gruber: https://daringfireball.net/
 .. _Chad Miller: http://web.chad.org/
 
 .. _Pyblosxom: http://roughingit.subtlehints.net/pyblosxom
-.. _SmartyPants: http://daringfireball.net/projects/smartypants/
+.. _SmartyPants: https://daringfireball.net/projects/smartypants/
 .. _Movable Type: http://www.movabletype.org/
 
 """
@@ -887,7 +887,8 @@ def _tokenize(str):
 
     return tokens
 
-def run_tests():
+
+def run_tests(return_tests=False):
     import unittest
     sp = smartyPants
 
@@ -929,6 +930,8 @@ def run_tests():
             self.assertEqual(sp('''"Isn't this fun?"'''), '''&#8220;Isn&#8217;t this fun?&#8221;''')
 
     tests = unittest.defaultTestLoader.loadTestsFromTestCase(TestSmartypantsAllAttributes)
+    if return_tests:
+        return tests
     unittest.TextTestRunner(verbosity=4).run(tests)
 
 
